// DiscoverServer.h: interface for the CDiscoverServer class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DISCOVERSERVER_H__8806AF5F_8192_4153_B3BD_19212808F4AE__INCLUDED_)
#define AFX_DISCOVERSERVER_H__8806AF5F_8192_4153_B3BD_19212808F4AE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/* UDP port numbers for Discover */
#define	DISCOVER_SERVER_PORT 48321	/* PC client Tx port, SDR Server Rx Port */
#define DISCOVER_CLIENT_PORT 48322	/* PC client Rx port, SDR Server Tx Port */


typedef struct  _DISCOVER_MSG_SDRXX
{		//56 fixed common byte fields
	unsigned char length[2]; 	//length of total message in bytes (little endian byte order)
	unsigned char key[2];		//fixed key key[0]==0x5A  key[1]==0xA5
	unsigned char op; 		//0==Request(to device)  1==Response(from device) 2 ==Set(to device)
	char name[16];			//Device name string null terminated
	char sn[16];			//Serial number string null terminated
	unsigned char ipaddr[16];	//device IP address (little endian byte order)
	unsigned char port[2];		//device Port number (little endian byte order)
	unsigned char customfield;	//Specify a custom data field for a particular device
		//start of optional variable custom byte fields
	unsigned char fwver[2];		//Firmware version*100 (little endian byte order)(read only)
	unsigned char btver[2];		//Boot version*100 (little endian byte order) (read only)
	unsigned char subnet[4];	//IP subnet mask (little endian byte order)
	unsigned char gwaddr[4];	//gateway address (little endian byte order)
	char connection[32];		//interface connection string null terminated(ex: COM3, DEVTTY5, etc)
	unsigned char status;		//bit 0 == TCP connected   Bit 1 == running  Bit 2-7 not defined
	unsigned char future[15];	//future use
}tDiscover_MSG;

#define STATUS_BIT_CONNECTED (1)
#define STATUS_BIT_RUNNING (2)


class CDiscoverServer  
{
public:
	CDiscoverServer();
	virtual ~CDiscoverServer();

	int m_IPAdr;
	unsigned short m_Port;
	CString m_SdrName;
	CString m_SdrSN;

	unsigned short m_BootVer;
	unsigned short m_FirmVer;

	BOOL m_InUse;


	void Stop();
	void Start();


	void UDPThread();

private:
	static UINT UDPThreadProc(LPVOID pParam);	//must be static since is thread
	BOOL m_UDPThreadRunning;
	CWinThread *m_pUDPThread;


	void CloseSockets();
	void SetupFDSets(fd_set& ReadFDs,  fd_set& ExceptFDs);
	void ParseDiscover( char* Buf, int Len );
	void SetSettings(void);
	void SendSettings(void);
	void SendDiscoverData(tDiscover_MSG* msg );


	BOOL m_RequestActive;
	CString m_Str;
	tDiscover_MSG m_Discovermsg;
	SOCKET m_hRxSocket;
	SOCKET m_hTxSocket;

};

#endif // !defined(AFX_DISCOVERSERVER_H__8806AF5F_8192_4153_B3BD_19212808F4AE__INCLUDED_)
