// USBsocket.cpp: implementation of the CUSBsocket class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SDRxxServer.h"
#include "ascpdefs.h"
#include "USBsocket.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// local defines
#define MSGSTATE_HDR1 0		//ASCP msg parsing states
#define MSGSTATE_HDR2 1
#define MSGSTATE_DATA 2

#define LENGTH_MASK 0x1FFF	// mask for message length

#define TYPE_HOST_SET_CITEM (0<<5)		//message to shut of SDR14
#define CI_RX_STATE 0x0018
 #define CI_RX_STATE_REQLEN 5
 #define CI_RX_STATE_SETRESPLEN 8
  #define RX_STATE_OFF 0x00
  #define RX_STATE_IDLE 0x01
  #define RX_STATE_ON 0x02


//USB status
#define USB_NOT_RUNNING 0
#define USB_NOT_CONNECTED 1
#define USB_CONNECTED 2
#define USB_BUSY 3

#define USB_TIMEOUT 500	//USB activity timeout

#define SOCKET_BUSY 4

#define MAX_RX 8194

#define RFSAMPLER_NAME "RF Sampler"
#define SDR14_NAME "SDR-14"
#define SDRIQ_NAME "SDR-IQ"

#define MAX_USB_XFER_SIZE (128*64)		//must be multiple of 64 and pwr of 2 ??? else close() hangs
#define MAX_USB_BUF_SIZE 65536
#define USB_FIFOSIZE 0x100000		//must be power of 2 for wrap around


#define KEEPALIVE_LIMIT 800

#define IOEEROM_SAMPLERATE 0	//External IO EEROM memory address definitions for sdriq
#define NOMINAL_ADFREQ 66666667


typedef FT_STATUS (WINAPI *PtrToOpen)(PVOID, FT_HANDLE *); 
	static PtrToOpen g_pOpen; 

typedef FT_STATUS (WINAPI *PtrToOpenEx)(PVOID, DWORD, FT_HANDLE *); 
	static PtrToOpenEx g_pOpenEx; 

typedef FT_STATUS (WINAPI *PtrToListDevices)(PVOID, PVOID, DWORD);
	static PtrToListDevices g_pListDevices; 

typedef FT_STATUS (WINAPI *PtrToGetStatus)(FT_HANDLE,LPDWORD,LPDWORD,LPDWORD);
	static PtrToGetStatus g_pGetStatus; 

typedef FT_STATUS (WINAPI *PtrToClose)(FT_HANDLE);
	static PtrToClose g_pClose;

typedef FT_STATUS (WINAPI *PtrToRead)(FT_HANDLE, LPVOID, DWORD, LPDWORD);
	static PtrToRead g_pRead;

typedef FT_STATUS (WINAPI *PtrToWrite)(FT_HANDLE, LPVOID, DWORD, LPDWORD);
	static PtrToWrite g_pWrite;

typedef FT_STATUS (WINAPI *PtrToResetDevice)(FT_HANDLE);
	static PtrToResetDevice g_pResetDevice;
	
typedef FT_STATUS (WINAPI *PtrToResetPort)(FT_HANDLE);
	static PtrToResetPort g_pResetPort;
	
typedef FT_STATUS (WINAPI *PtrToPurge)(FT_HANDLE, ULONG);
	static PtrToPurge g_pPurge;
	
typedef FT_STATUS (WINAPI *PtrToSetTimeouts)(FT_HANDLE, ULONG, ULONG);
	static PtrToSetTimeouts g_pSetTimeouts;

typedef FT_STATUS (WINAPI *PtrToGetQueueStatus)(FT_HANDLE, LPDWORD);
	static PtrToGetQueueStatus g_pGetQueueStatus;

typedef FT_STATUS (WINAPI *PtrToSetEventNotification)(FT_HANDLE, DWORD,PVOID);
	static PtrToSetEventNotification g_pSetEventNotification;

typedef FT_STATUS (WINAPI *PtrToSetLatencyTimer)(FT_HANDLE, UCHAR);
	static PtrToSetLatencyTimer g_pSetLatencyTimer;

typedef FT_STATUS (WINAPI *PtrToFT_EE_Read)(FT_HANDLE, PFT_PROGRAM_DATA);
	static PtrToFT_EE_Read g_pFT_EE_Read;

typedef FT_STATUS (WINAPI *PtrFT_SetResetPipeRetryCount)(FT_HANDLE, DWORD);
	static PtrFT_SetResetPipeRetryCount g_pFT_SetResetPipeRetryCount;
	
typedef FT_STATUS (WINAPI *PtrFT_SetUSBParameters)( FT_HANDLE ftHandle, ULONG ulInTransferSize, ULONG ulOutTransferSize);
	static PtrFT_SetUSBParameters g_pFT_SetUSBParameters;
	
typedef FT_STATUS (WINAPI *PtrFT_EE_UARead)( FT_HANDLE ftHandle, PUCHAR pucData, DWORD dwDataLen,LPDWORD lpdwBytesRead);
	static PtrFT_EE_UARead g_pFT_EE_UARead;

typedef FT_STATUS (WINAPI *PtrFT_EE_UAWrite)( FT_HANDLE ftHandle, PUCHAR pucData, DWORD dwDataLen);
	static PtrFT_EE_UAWrite g_pFT_EE_UAWrite;

typedef FT_STATUS (WINAPI *PtrFT_GetDeviceInfo)(FT_HANDLE ftHandle,  FT_DEVICE *pftType, LPDWORD lpdwID,
											PCHAR pcSerialNumber, PCHAR pcDescription,PVOID pvDummy);
static PtrFT_GetDeviceInfo g_pFT_GetDeviceInfo;



static int g_Counter = 0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CUSBsocket::CUSBsocket()
{
	m_USBPortOpen = FALSE;
	m_USBThreadRunning = FALSE;
	m_FIFOThreadRunning = FALSE;
	m_SocketThreadRunning = FALSE;
	m_USBReInit = FALSE;
	m_hReadEvent = NULL;
	m_hmodule = NULL;
	m_pSocketThread = NULL;
	m_pUSBThread = NULL;
	m_UsbSstatus = USB_NOT_RUNNING;
	m_SocketSstatus = SOCKET_NOT_RUNNING;
	m_pMsgQue = NULL;
	m_hFifoWriteEvent = NULL;
	m_pUSBFifo = NULL;
	m_pUSBBuffer = NULL;
	m_pUSBFifoThread = NULL;
	m_KeepAlive = 0;
	m_FiFoReset = FALSE;
	m_SdrSN = "";
	m_SdrName = "";
	m_BootVer = 0;
	m_FirmVer = 0;
	m_ClientSocket = INVALID_SOCKET;
	m_ListenSocket = INVALID_SOCKET;
	m_pMsgQue = (MSG_STRUC*) new MSG_STRUC[QUEUE_SIZE];

	InitializeCriticalSection(&m_SockCriticalSection);
	InitializeCriticalSection(&m_USBCriticalSection);
	for(int i=0; i<EEROM_SIZE; i++)
		m_EEROMData[EEROM_SIZE] = 0;

}

CUSBsocket::~CUSBsocket()
{
	StopControl();
	DeleteCriticalSection(&m_SockCriticalSection);
	DeleteCriticalSection(&m_USBCriticalSection);
	if(m_pMsgQue)
	{
		delete m_pMsgQue;
		m_pMsgQue = NULL;
	}
}


//////////////////////////////////////////////////////////////////////
//  Starts up both USB and Socket worker threads
//////////////////////////////////////////////////////////////////////
void CUSBsocket::StartControl(u_short PortNum, CString  SNFilterStr) 
{
	StopControl();	//if thread is already running stop it
	m_PortNum = PortNum;
	m_SNFilterStr = SNFilterStr;
	//Start up USB worker thread
	m_pUSBThread = AfxBeginThread(USBThreadProc,
								this,
								THREAD_PRIORITY_TIME_CRITICAL,
								0,
								CREATE_SUSPENDED);
	m_pUSBThread->m_bAutoDelete = FALSE;
	m_USBThreadRunning = TRUE;
	m_pUSBThread->ResumeThread();
	//Start up Socket Worker thread
	m_pSocketThread = AfxBeginThread(SocketThreadProc,
								this,
								THREAD_PRIORITY_HIGHEST,
								0,
								CREATE_SUSPENDED);
	m_pSocketThread->m_bAutoDelete = FALSE;
	m_SocketThreadRunning = TRUE;
	m_pSocketThread->ResumeThread();

}


//////////////////////////////////////////////////////////////////////
//Stops all worker threads and shuts down USB and network
//////////////////////////////////////////////////////////////////////
void CUSBsocket::StopControl() 
{
	StopSDRxx();
	Sleep(100);		//give it time to get sent
	//tell threads to stop
TRACE("\n\rStop");
	m_SocketThreadRunning = FALSE;
	if(m_pSocketThread)
	{
		m_pSocketThread->ResumeThread();
		::WaitForSingleObject(m_pSocketThread->m_hThread, INFINITE);
		delete m_pSocketThread;
		m_pSocketThread = NULL;
	}
TRACE("\n\rSocket Stopped");
	m_USBThreadRunning = FALSE;
	if(m_pUSBThread)
	{
//		m_pUSBThread->ResumeThread();
		::WaitForSingleObject(m_pUSBThread->m_hThread, INFINITE);
		delete m_pUSBThread;
		m_pUSBThread = NULL;
	}
	m_UsbSstatus = USB_NOT_RUNNING;
	m_SocketSstatus = SOCKET_NOT_RUNNING;
TRACE("\n\rUSB Stopped");
}

//////////////////////////////////////////////////////////////////////
// Worker Thread static procedure that just calls the main USB process loop
//////////////////////////////////////////////////////////////////////
UINT CUSBsocket::USBThreadProc(LPVOID pParam)
{
CUSBsocket* pObject = (CUSBsocket*)pParam;
	pObject->USBThread();
	return 10;
}

//////////////////////////////////////////////////////////////////////
// Worker Thread static procedure that just calls the main Socket process loop
//////////////////////////////////////////////////////////////////////
UINT CUSBsocket::SocketThreadProc(LPVOID pParam)
{
CUSBsocket* pObject = (CUSBsocket*)pParam;
	pObject->SocketThread();
	return 11;
}

//////////////////////////////////////////////////////////////////////
// Worker Thread static procedure that just calls the FIFO process loop
//////////////////////////////////////////////////////////////////////
UINT CUSBsocket::USBFifoThreadProc(LPVOID pParam)
{
CUSBsocket* pObject = (CUSBsocket*)pParam;
	pObject->FifoThread();
	return 12;
}


//////////////////////////////////////////////////////////////////////
// return current USB status
//////////////////////////////////////////////////////////////////////
CString CUSBsocket::GetUSBstatus(BOOL &Connected)
{
	Connected = FALSE;
	switch(m_UsbSstatus)
	{
		case USB_NOT_RUNNING:
			m_Str = "Server Off";
			break;
		case USB_NOT_CONNECTED:
			m_Str = "SDR-xx Not Connected";
			break;
		case USB_CONNECTED:
			m_Str = m_SdrName + " "  + m_SdrSN + " Connected";
			Connected = TRUE;
			break;
		case USB_BUSY:
			m_Str = m_SdrName + " "  + m_SdrSN +  " Running";
			Connected = TRUE;
			break;
	}
	return m_Str;
}

//////////////////////////////////////////////////////////////////////
// return current Socket status
//////////////////////////////////////////////////////////////////////
CString CUSBsocket::GetSocketstatus()
{
	switch(m_SocketSstatus)
	{
		case SOCKET_NOT_RUNNING:
			m_Str = "Server Off";
			break;
		case SOCKET_WAITINGUSB:
			m_Str = "Server Paused";
			break;
		case SOCKET_WAITINGCONNECT:
			m_Str = "Waiting For Connect";
			break;
		case SOCKET_CONNECTED:
			m_Str = "Connected to ";
			m_Str += inet_ntoa(m_ClientTCPAddress.sin_addr);
			break;
		case SOCKET_BUSY:
			m_Str = "Sending Data";
			break;
	}
	return m_Str;
}

//////////////////////////////////////////////////////////////////////
//  Called to open the USB dll and assign functions
// Not all the FTDI dll functions are used.
//////////////////////////////////////////////////////////////////////
BOOL CUSBsocket::OpenUSBdll()
{
	m_hmodule = LoadLibrary("Ftd2xx.dll");	
	if(m_hmodule == NULL)
		return FALSE;
	g_pWrite = (PtrToWrite)GetProcAddress(m_hmodule, "FT_Write");
	if(g_pWrite == NULL)
		return FALSE;
	g_pRead = (PtrToRead)GetProcAddress(m_hmodule, "FT_Read");
	if(g_pRead == NULL)
		return FALSE;
	g_pOpen = (PtrToOpen)GetProcAddress(m_hmodule, "FT_Open");
	if(g_pOpen == NULL)
		return FALSE;
	g_pOpenEx = (PtrToOpenEx)GetProcAddress(m_hmodule, "FT_OpenEx");
	if(g_pOpenEx == NULL)
		return FALSE;
	g_pListDevices = (PtrToListDevices)GetProcAddress(m_hmodule, "FT_ListDevices");
	if(g_pListDevices == NULL)
		return FALSE;
	g_pClose = (PtrToClose)GetProcAddress(m_hmodule, "FT_Close");
	if(g_pClose == NULL)
		return FALSE;
	g_pResetDevice = (PtrToResetDevice)GetProcAddress(m_hmodule, "FT_ResetDevice");
	if(g_pResetDevice == NULL)
		return FALSE;
	g_pResetPort = (PtrToResetDevice)GetProcAddress(m_hmodule, "FT_ResetPort");
	if(g_pResetPort == NULL)
		return FALSE;
	g_pPurge = (PtrToPurge)GetProcAddress(m_hmodule, "FT_Purge");
	if(g_pPurge == NULL)
		return FALSE;
	g_pSetTimeouts = (PtrToSetTimeouts)GetProcAddress(m_hmodule, "FT_SetTimeouts");
	if(g_pSetTimeouts == NULL)
		return FALSE;
	g_pGetQueueStatus = (PtrToGetQueueStatus)GetProcAddress(m_hmodule, "FT_GetQueueStatus");
	if(g_pGetQueueStatus == NULL)
		return FALSE;
	g_pGetStatus = (PtrToGetStatus)GetProcAddress(m_hmodule, "FT_GetStatus");
	if(g_pGetStatus == NULL)
		return FALSE;
	g_pSetEventNotification = (PtrToSetEventNotification)GetProcAddress(m_hmodule, "FT_SetEventNotification");
	if(g_pSetEventNotification == NULL)
		return FALSE;
	g_pSetLatencyTimer = (PtrToSetLatencyTimer)GetProcAddress(m_hmodule, "FT_SetLatencyTimer");
	if(g_pSetLatencyTimer == NULL)
		return FALSE;
	g_pFT_EE_Read = (PtrToFT_EE_Read)GetProcAddress(m_hmodule, "FT_EE_Read");
	if(g_pFT_EE_Read == NULL)
		return FALSE;
	g_pFT_SetResetPipeRetryCount = (PtrFT_SetResetPipeRetryCount)GetProcAddress(m_hmodule, "FT_SetResetPipeRetryCount");
	if(g_pFT_SetResetPipeRetryCount == NULL)
		return FALSE;
	g_pFT_SetUSBParameters = (PtrFT_SetUSBParameters)GetProcAddress(m_hmodule, "FT_SetUSBParameters");
	if(g_pFT_SetUSBParameters == NULL)
		return FALSE;
	g_pFT_EE_UARead = (PtrFT_EE_UARead)GetProcAddress(m_hmodule, "FT_EE_UARead");
	if(g_pFT_EE_UARead == NULL)
		return FALSE;
	g_pFT_EE_UAWrite = (PtrFT_EE_UAWrite)GetProcAddress(m_hmodule, "FT_EE_UAWrite");
	if(g_pFT_EE_UAWrite == NULL)
		return FALSE;
	g_pFT_GetDeviceInfo = (PtrFT_GetDeviceInfo)GetProcAddress(m_hmodule, "FT_GetDeviceInfo");
	if(g_pFT_GetDeviceInfo == NULL)
		return FALSE;
	return TRUE;
}


void CUSBsocket::ReadIOMemory( int adr, unsigned int* pData)
{
unsigned int val = 0;
	if( m_FTStatus != FT_OK)
		return;
	val = (unsigned int)m_EEROMData[adr];
	val = val + ( (unsigned int)m_EEROMData[adr+1] << 8);
	val = val + ( (unsigned int)m_EEROMData[adr+2] << 16);
	val = val + ( (unsigned int)m_EEROMData[adr+3] << 24);
	if( (val<50000000) || (val>70000000) )
	{
		val = NOMINAL_ADFREQ;
		m_EEROMData[adr] = (unsigned char)(val & 0xFF);
		m_EEROMData[adr+1] = (unsigned char)((val>>8) & 0xFF);
		m_EEROMData[adr+2] = (unsigned char)((val>>16) & 0xFF);
		m_EEROMData[adr+3] = (unsigned char)((val>>24) & 0xFF);
		FT_STATUS Status = g_pFT_EE_UAWrite(m_FTHandle, m_EEROMData, EEROM_SIZE);
		Sleep(100);
	}
	pData[0] = val;
}

void CUSBsocket::WriteIOMemory( int adr, unsigned int Data)
{
unsigned int val;
FT_STATUS Status;
	if( m_FTStatus != FT_OK)
		return;
	if( m_FTStatus == FT_OK)
	{
		ReadIOMemory( adr, &val);	//read present value
		if(val != Data )	//need to write EERom
		{
			m_EEROMData[adr] = (unsigned char)(Data & 0xFF);
			m_EEROMData[adr+1] = (unsigned char)((Data>>8) & 0xFF);
			m_EEROMData[adr+2] = (unsigned char)((Data>>16) & 0xFF);
			m_EEROMData[adr+3] = (unsigned char)((Data>>24) & 0xFF);
			Status = g_pFT_EE_UAWrite(m_FTHandle, m_EEROMData, EEROM_SIZE);
			Sleep(100);
		}
	}
}

void CUSBsocket::SetSDRIQSampleRateFromEEROM(unsigned int rate)
{
unsigned char msg[10];
DWORD txbytes;
	if( m_FTStatus != FT_OK)
		return;
	if( (rate>50000000) && (rate<70000000) )
	{
		//create ASCP msg
		msg[0] = CI_RX_IN_SAMPLE_RATE_SETRESPLEN;
		msg[1] = TYPE_HOST_SET_CITEM;
		msg[2] = (unsigned char)CI_RX_IN_SAMPLE_RATE;
		msg[3] = 0;
		msg[4] = 0;
		msg[5] = (unsigned char)((rate>>0)&0xFF);
		msg[6] = (unsigned char)((rate>>8)&0xFF);
		msg[7] = (unsigned char)((rate>>16)&0xFF);
		msg[8] = (unsigned char)((rate>>24)&0xFF);
		EnterCriticalSection(&m_USBCriticalSection);
		m_FTStatus = g_pWrite( m_FTHandle, (LPVOID)msg, CI_RX_IN_SAMPLE_RATE_SETRESPLEN, &txbytes);
		LeaveCriticalSection(&m_USBCriticalSection);
	}
}

//////////////////////////////////////////////////////////////////////
//  Main processing loop of this USB Worker Thread
// This is a worker thread that services the USB port.
//////////////////////////////////////////////////////////////////////
void CUSBsocket::USBThread()
{
DWORD eventstat;
DWORD RxBytes;
DWORD TxBytes;
DWORD bytesGot;
FT_DEVICE ftDevice;
DWORD deviceID;
char SerialNumber[16];
char Description[64];
CString descr;
int WaitTimer;
unsigned int IQSampleRate=0;
	OpenUSBdll();	//open the FTDI driver dll
	m_hReadEvent = CreateEvent( NULL, FALSE, FALSE, NULL );
	m_hFifoWriteEvent = CreateEvent(NULL, FALSE,FALSE, NULL);
	//allocate buffer memory for USB and FIFO
	if(!m_pUSBFifo)
		m_pUSBFifo = new unsigned __int8[USB_FIFOSIZE];
	if(!m_pUSBBuffer)
		m_pUSBBuffer = new unsigned __int8[MAX_USB_BUF_SIZE];
	//start USB FIFO worker thread
	m_pUSBFifoThread = AfxBeginThread(USBFifoThreadProc,
								this,
								THREAD_PRIORITY_HIGHEST,
								0,
								CREATE_SUSPENDED);
	m_pUSBFifoThread->m_bAutoDelete = FALSE;
	m_FIFOThreadRunning = TRUE;
	m_pUSBFifoThread->ResumeThread();

	m_FifoHead = 0;
	m_FifoTail = 0;
	while(m_USBThreadRunning)
	{
		if(!m_USBPortOpen )	//open USB port if not open already
		{
			if(m_hmodule)
			{
				if(m_SNFilterStr != "")
				{	//try to open an FTDI device with specified serial number string
					m_FTStatus = g_pOpenEx((PCHAR)(LPCTSTR)m_SNFilterStr, FT_OPEN_BY_SERIAL_NUMBER, &m_FTHandle);
					if( m_FTStatus == FT_OK)
					{	//found one so now need to figure out if sdr-iq or sdr-14
						m_FTStatus = g_pFT_GetDeviceInfo(m_FTHandle, &ftDevice, &deviceID, SerialNumber, Description,NULL);
						descr = Description;
						if(descr == SDR14_NAME)
						{
							m_SdrName = "SDR-14";
						}
						else if(descr == SDRIQ_NAME)
						{
							m_SdrName = "SDR-IQ";
							g_pSetLatencyTimer(m_FTHandle,2);
							//since SDR-IQ cant read its own FTDI EEROM data, need to read it here and send it back
							Sleep(100);
							g_pFT_EE_UARead(m_FTHandle, m_EEROMData, EEROM_SIZE, &bytesGot);	//read sdriq eerom params
							ReadIOMemory( IOEEROM_SAMPLERATE, &IQSampleRate);	//get sample rate from EEROM data
							SetSDRIQSampleRateFromEEROM(IQSampleRate);			//send it back to SDR-IQ
//TRACE("\nGet IQ Sample Rate = %d %s", IQSampleRate, Description);
							Sleep(500);
							m_FiFoReset = FALSE;
							m_SeqNumber = 0;
						}

					}
				}
				else
				{
					m_FTStatus = g_pOpenEx(SDR14_NAME, FT_OPEN_BY_DESCRIPTION, &m_FTHandle);
					if( m_FTStatus == FT_OK)
					{
						m_SdrName = "SDR-14";
					}
					else
					{
						m_FTStatus = g_pOpenEx(SDRIQ_NAME, FT_OPEN_BY_DESCRIPTION, &m_FTHandle);
						if( m_FTStatus == FT_OK)
						{
							m_SdrName = "SDR-IQ";
							g_pSetLatencyTimer(m_FTHandle,2);
							//since SDR-IQ cant read its own FTDI EEROM data, need to read it here and send it back
							Sleep(100);
							g_pFT_EE_UARead(m_FTHandle, m_EEROMData, EEROM_SIZE, &bytesGot);	//read sdriq eerom params
							ReadIOMemory( IOEEROM_SAMPLERATE, &IQSampleRate);	//get sample rate from EEROM data
							SetSDRIQSampleRateFromEEROM(IQSampleRate);			//send it back to SDR-IQ
//TRACE("\nGet IQ Sample Rate = %d", IQSampleRate);
							Sleep(500);
							m_FiFoReset = FALSE;
							m_SeqNumber = 0;
						}
						else
						{
							m_FTStatus = FT_DEVICE_NOT_FOUND;
						}
					}
				}
			}
			if( m_FTStatus != FT_OK)
			{	// error so must not be connected
				m_UsbSstatus = USB_NOT_CONNECTED;
				WaitTimer = 10;
				while( (--WaitTimer>0) && m_USBThreadRunning)
					Sleep(100);		//wait 1 second before trying again
			}
			else
			{		//Here if SDR-xx was found and USB port opened OK
				m_USBPortOpen = TRUE;
				m_MsgState = MSGSTATE_HDR1;
				m_RxMsgIndex = 0;
				m_RxMsgLength = 0;
				m_TCPMsgState = MSGSTATE_HDR1;
				m_TCPMsgIndex = 0;
				m_TCPMsgLength = 0;
				g_pFT_SetUSBParameters(m_FTHandle, MAX_USB_XFER_SIZE, 0);
				g_pResetDevice(m_FTHandle);
				g_pFT_SetResetPipeRetryCount(m_FTHandle, 50);
				g_pPurge(m_FTHandle,FT_PURGE_RX|FT_PURGE_TX);
				g_pSetTimeouts(m_FTHandle, 1500, 1500);
				g_pGetQueueStatus(m_FTHandle,&RxBytes);
				//flush out anything the SDR-xx may have buffered up
				if(RxBytes>0)
				{
					EnterCriticalSection(&m_USBCriticalSection);
					m_FTStatus = g_pRead(m_FTHandle, m_pUSBBuffer,	RxBytes, &bytesGot);
					LeaveCriticalSection(&m_USBCriticalSection);
				}
				RxBytes=0;
				m_Qhead = 0;
				m_Qtail = 0;
				m_FifoHead = 0;
				m_FifoTail = 0;
				m_UsbSstatus = USB_CONNECTED;
				//setup FTDI read event
				m_FTStatus = g_pSetEventNotification(m_FTHandle,FT_EVENT_RXCHAR,m_hReadEvent);
				if( m_FTStatus != FT_OK)
				{	// error so must not be connected
					m_UsbSstatus = USB_NOT_CONNECTED;
					Sleep(100);
				}
				RequestDeviceInfo();
			}
		}
		else	//Here with USB port open and ready for action
		{
			//if no data to read just wait for USB read or an event timeout
			eventstat = WaitForSingleObject( m_hReadEvent, USB_TIMEOUT);
			if(eventstat == WAIT_TIMEOUT)
			{
				m_FTStatus = g_pGetQueueStatus(m_FTHandle, &RxBytes);
				if( m_FTStatus != FT_OK)
				{	//if error
					m_FTStatus = g_pClose(m_FTHandle);
					m_USBPortOpen = FALSE;
					RxBytes = 0;
				}
				else
				{
					m_UsbSstatus = USB_CONNECTED;
					//message timeout so reset pointers
					m_MsgState = MSGSTATE_HDR1;
					m_RxMsgIndex = 0;		//reset msg decode state pointers
					m_RxMsgLength = 0;
					m_TCPMsgState = MSGSTATE_HDR1;
					m_TCPMsgIndex = 0;
					m_TCPMsgLength = 0;
					m_FifoHead = 0;
					m_FifoTail = 0;
				}
//				TRACE("\nUSB Timeout");
			}
			else
			{
				m_FTStatus = g_pGetStatus(m_FTHandle, &RxBytes, &TxBytes, &eventstat);
				if( m_FTStatus != FT_OK)
				{	//if error
					m_FTStatus = g_pClose(m_FTHandle);
					m_USBPortOpen = FALSE;
					RxBytes = 0;
				}
				else	//here if event triggered or timeout
				{
					if(RxBytes>0)	//must be data in USB Rx Queue
					{
						EnterCriticalSection(&m_USBCriticalSection);
						m_FTStatus = g_pRead(m_FTHandle, m_pUSBBuffer,	RxBytes, &bytesGot); //read data
						LeaveCriticalSection(&m_USBCriticalSection);
						if( m_FTStatus != FT_OK)
						{	//if error, bail out
							m_FTStatus = g_pClose(m_FTHandle);
							m_USBPortOpen = FALSE;
							break;
						}
						else
						{	// process the bytes received so far.
							EnterCriticalSection(&m_USBCriticalSection);
							for(unsigned int i=0; i<bytesGot; i++) //Place in FIFO
							{
								m_pUSBFifo[m_FifoHead++] = m_pUSBBuffer[i];
								m_FifoHead &= (USB_FIFOSIZE-1);
							}
							::SetEvent(m_hFifoWriteEvent);
							LeaveCriticalSection(&m_USBCriticalSection);
						}
					}
					else	//There is no data in USB RX queue 
					{
						Sleep(0);	//shouldnt get here but...
					}
				}
			}
		}
		if(m_USBReInit)
		{
			if(m_USBPortOpen )
			{
				m_USBPortOpen = FALSE;
				m_FTStatus = g_pClose(m_FTHandle);
			}
			m_USBReInit = FALSE;
		}
	}
	if(m_USBPortOpen  && ( m_FTStatus == FT_OK) )
	{
		m_FTStatus = g_pClose(m_FTHandle);
		m_USBPortOpen = FALSE;
	}
TRACE("\nUSB closed");
	if(m_pUSBBuffer)
		delete m_pUSBBuffer;
	m_pUSBBuffer = NULL;
	//stop and delete FIFO thread and its buffer
	if(m_pUSBFifoThread)
	{
		m_FIFOThreadRunning = FALSE;
		SetEvent(m_hFifoWriteEvent);
		m_pUSBFifoThread->ResumeThread();
		::WaitForSingleObject(m_pUSBFifoThread->m_hThread, INFINITE);
		delete m_pUSBFifoThread;
		m_pUSBFifoThread = NULL;
		if(m_pUSBFifo)
			delete m_pUSBFifo;
		m_pUSBFifo = NULL;
	}
	if(m_hFifoWriteEvent)
	{
		CloseHandle(m_hFifoWriteEvent);
		m_hFifoWriteEvent = NULL;
	}
	if(m_hReadEvent)
	{
		CloseHandle(m_hReadEvent);
		m_hReadEvent = NULL;
	}
	if(m_hmodule)
	{
		FreeLibrary(m_hmodule);
		m_hmodule = NULL;
	}
}


//////////////////////////////////////////////////////////////////////
//  Main processing loop of worker thread for removing USB FIFO data
//This thread also createes the UDP socket for I/Q data transfer
//////////////////////////////////////////////////////////////////////
void CUSBsocket::FifoThread()
{
DWORD eventstat;
unsigned __int8 data;
int nRet;
BOOL FifoEmpty = TRUE;
	m_FifoHead = 0;
	m_FifoTail = 0;
	m_UDPSocket = INVALID_SOCKET;
	if( (m_UDPSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP) ) == INVALID_SOCKET)
	{
		nRet = ::WSAGetLastError();
		TRACE("Can't Open UDPSocket %d\n", nRet);
		m_USBReInit = TRUE;	//signal USB port so it can reinitialize
	}
	int buffsize = 1000000;
	nRet = setsockopt(m_UDPSocket, SOL_SOCKET, SO_RCVBUF, (const char*)&buffsize, sizeof(buffsize));
	while(m_FIFOThreadRunning)
	{
		eventstat = WaitForSingleObject( m_hFifoWriteEvent, 1);
		EnterCriticalSection(&m_USBCriticalSection);
		FifoEmpty = (m_FifoHead == m_FifoTail);
		LeaveCriticalSection(&m_USBCriticalSection);
		while( !FifoEmpty)	//process until all bytes read from USB Fifo
		{
			EnterCriticalSection(&m_USBCriticalSection);
			data = m_pUSBFifo[m_FifoTail++];
			m_FifoTail &= (USB_FIFOSIZE-1);
			FifoEmpty = (m_FifoHead == m_FifoTail);
			LeaveCriticalSection(&m_USBCriticalSection);
			switch(m_MsgState)	//Simple state machine to get generic ASCP msg
			{
				case MSGSTATE_HDR1:		//get first header byte of ASCP msg
					m_rxASCPmsg.bytes.b0 = data;
					m_MsgState = MSGSTATE_HDR2;	//go to get second header byte state
					break;
				case MSGSTATE_HDR2:	//here for second byte of header
					m_rxASCPmsg.bytes.b1 = data;
					m_RxMsgLength = m_rxASCPmsg.words.header&0x1FFF;
					m_RxMsgIndex = 2;
					if(2 == m_RxMsgLength)	//if msg has no parameters then we are done
					{
						m_MsgState = MSGSTATE_HDR1;	//go back to first state
						SendTCPMsgToSocket( &m_rxASCPmsg, m_RxMsgLength);
					}
					else	//there are data bytes to fetch
					{
						if( 0 == m_RxMsgLength)
							m_RxMsgLength = 8192+2;	//handle special case of 8192 byte data message
						m_MsgState = MSGSTATE_DATA;	//go to data byte reading state
					}
					break;
				case MSGSTATE_DATA:	//try to read the rest of the message or to end of USB buffer
					m_rxASCPmsg.buf[m_RxMsgIndex++] = data;
					if( m_RxMsgIndex >= m_RxMsgLength )
					{	
						m_MsgState = MSGSTATE_HDR1;	//go back to first stage
						if( 0x8000 == m_rxASCPmsg.words.header)	//if an sdr-xx IQ data msg then use UDP
						{
							m_UsbSstatus = USB_BUSY;
							SendUDPMsgToSocket( &m_rxASCPmsg, m_RxMsgLength);
						}
						else
						{	//if TCP not connected then decode msgs locally
							if(	m_SocketSstatus != SOCKET_CONNECTED)
								InterceptSdrMsgs(&m_rxASCPmsg, m_RxMsgLength);	//call to intercept some msgs we need for local processing
							else
								SendTCPMsgToSocket( &m_rxASCPmsg, m_RxMsgLength);
						}
					}

					break;
			} //end switch statement
		} //end while loop
	}
	if(m_UDPSocket != INVALID_SOCKET)
		closesocket(m_UDPSocket);
}


//////////////////////////////////////////////////////////////////////
//  Called to send data to the SDR-xx using the USB port
//////////////////////////////////////////////////////////////////////
void CUSBsocket::SendDataToUSB( char* pData, int length)
{
DWORD txbytes;
int timeout = 100;	//xmit timeout ~1 second
DWORD bytesremaining = length;
	if(!m_USBPortOpen)
		return;
	AssembleASCPMsg(pData, length);	//assemble/Parse it to intercept some msgs
	while( (bytesremaining > 0) && m_USBPortOpen)
	{
		EnterCriticalSection(&m_USBCriticalSection);
		m_FTStatus = g_pWrite( m_FTHandle, (LPVOID)pData, bytesremaining, &txbytes);
		LeaveCriticalSection(&m_USBCriticalSection);
		bytesremaining -= txbytes;
		if(bytesremaining>0)	//if driver didn't send them all
		{
			Sleep(10);			//wait
			if(timeout-- <= 0)	//bail if taking too long
				m_FTStatus = FT_FAILED_TO_WRITE_DEVICE;
		}
		if( m_FTStatus != FT_OK)
		{
			m_FTStatus = g_pClose(m_FTHandle);
			m_USBPortOpen = FALSE;
			bytesremaining = 0;	//force exit of loop
		}
	}
}

//////////////////////////////////////////////////////////////////////
//  Called to parse raw TCP data stream from client message 
// to decode keep alive msg
//////////////////////////////////////////////////////////////////////
void CUSBsocket::AssembleASCPMsg( char* pData, int length )
{
int i = 0;
t32BITS tmp;
	while(length>0)
	{
		switch(m_TCPMsgState)
		{
			case MSGSTATE_HDR1:		//get first header byte of msg
				m_TCPASCPmsg.buf[0] = pData[i++];
				m_TCPMsgIndex = 1;
				m_TCPMsgState = MSGSTATE_HDR2;
				break;
			case MSGSTATE_HDR2:	//here for second byte of header
				m_TCPASCPmsg.buf[1] = pData[i++];
				m_TCPMsgIndex = 2;
				m_TCPMsgLength = (UINT)m_TCPASCPmsg.buf[0] + 256*(UINT)(m_TCPASCPmsg.buf[1]&0x1F);
				if(m_TCPMsgLength==2)	//if msg has no parameters then we are done
				{
					m_TCPMsgState = MSGSTATE_HDR1;
				}
				else	//are data bytes to fetch
				{
					if(m_TCPMsgLength==0)
						m_TCPMsgLength = 8192+2;
					m_TCPMsgState = MSGSTATE_DATA;
				}
				break;
			case MSGSTATE_DATA:
				m_TCPASCPmsg.buf[m_TCPMsgIndex++] = pData[i++];
				if( m_TCPMsgIndex >= m_TCPMsgLength )
				{	
//					if( (0x03==m_TCPASCPmsg.buf[0]) && (0x60==m_TCPASCPmsg.buf[1]) && (0x03==m_TCPASCPmsg.buf[2]) )
//					{	//Keepalive ASCP ack msg data item 3 back to connected SV client
//						SendTCPMsgToSocket( &m_TCPASCPmsg, 3);
//					}
//					else if( (0x03==m_TCPASCPmsg.buf[0]) && (0x60==m_TCPASCPmsg.buf[1]) && (0x00==m_TCPASCPmsg.buf[2]) )
//					{	//Keepalive ASCP ack msg data item 0 back to connected SV client
//						SendTCPMsgToSocket( &m_TCPASCPmsg, 3);
//					}
//					else if((CI_RX_IN_SAMPLE_RATE_SETRESPLEN==m_TCPASCPmsg.buf[0]) && 

					if((CI_RX_IN_SAMPLE_RATE_SETRESPLEN==m_TCPASCPmsg.buf[0]) && 
						(TYPE_HOST_SET_CITEM==m_TCPASCPmsg.buf[1]) && 
						(CI_RX_IN_SAMPLE_RATE==m_TCPASCPmsg.buf[2]) &&
						(0==m_TCPASCPmsg.buf[3]))
					{	//if is set Sample rate msg then need to write it to EEROM as well
						if(	m_SdrName == "SDR-IQ")	//only applies to SDR-IQ
						{
							tmp.bytes.b0 = m_TCPASCPmsg.buf[5];
							tmp.bytes.b1 = m_TCPASCPmsg.buf[6];
							tmp.bytes.b2 = m_TCPASCPmsg.buf[7];
							tmp.bytes.b3 = m_TCPASCPmsg.buf[8];
							WriteIOMemory( IOEEROM_SAMPLERATE, tmp.all);
//TRACE("\nSet IQ Sample Rate = %d", tmp.all);
						}
					}

					m_TCPMsgState = MSGSTATE_HDR1;
				}
				break;
		}
		length--;
	} //end while loop
}


//////////////////////////////////////////////////////////////////////
//  Called send an ASCP formated message to the TCP Client Socket
// it blocks for the send to complete or an error.
//////////////////////////////////////////////////////////////////////
void CUSBsocket::SendTCPMsgToSocket( ASCPstruct* pASCPmsg, int length)
{
int nBytesSent;
int nBytesToSend = length;

	if(	m_SocketSstatus != SOCKET_CONNECTED)
		return;
	if(m_ClientSocket == INVALID_SOCKET)
		return;
	if(!m_USBPortOpen)
		return;

//	strip out unsolicited RX_STATE msg since it bottles up the TCP stream
//  msg is 0x08 0x20 0x18 0x00  
//instead zero the UDP sequence number to indicate FIFO reset
	if(0x00182008 == pASCPmsg->longs.header)
	{
//	TRACE("\n\r Msg = %8.8X", pASCPmsg->longs.header);
		m_FiFoReset = TRUE;
		return;
	}

	while( nBytesToSend)
	{
		nBytesSent = send(m_ClientSocket, (const char*)pASCPmsg->buf, nBytesToSend, 0);
		if(nBytesSent == SOCKET_ERROR)
		{
			int err = WSAGetLastError();
			if(err != WSAEWOULDBLOCK)	//if not just an overlapped write
			{		// Something bad happened on the socket.  Deal with it.
				StopSDRxx();
				ShutdownConnection(m_ClientSocket);
				m_SocketSstatus = SOCKET_WAITINGCONNECT;
				m_USBReInit = TRUE;	//signal USB port so it can reinitialize
			}
			else
			{	//send did not complete but had no error so just wait and then try again
				Sleep(0);
			}
		}
		else
		{
			nBytesToSend -= nBytesSent;
		}
	}
}

//////////////////////////////////////////////////////////////////////
//  Called send an ASCP formated message to the Client UDP Socket
// it breaks the 8192 byte data packet into eight 1024 byte data packets
// for transport through UDP 
//////////////////////////////////////////////////////////////////////
void CUSBsocket::SendUDPMsgToSocket( ASCPstruct* pASCPmsg, int length)
{
	if(m_UDPSocket == INVALID_SOCKET)
		return;

	if(!m_USBPortOpen)
		return;

	if(	m_FiFoReset )
	{	//indicate fifo reset by zero sequence number
		m_FiFoReset = FALSE;;
		m_SeqNumber = 0;
	}
	//break 8192 byte packet into eight 1024 ASCP packets
	for(int i=2; i<8194; i+=1024)
		SendUDPPacketToSocket( pASCPmsg->buf+i );
	m_KeepAlive++;
	if( m_KeepAlive > g_Counter)
	{
		g_Counter = m_KeepAlive;
		TRACE("\nmax keepalive %d",g_Counter);
	}
	if(m_KeepAlive > KEEPALIVE_LIMIT)
	{
		m_KeepAlive = 0;
		StopSDRxx();
		ShutdownConnection(m_ClientSocket);
		m_SocketSstatus = SOCKET_WAITINGCONNECT;
		m_USBReInit = TRUE;	//signal USB port so it can reinitialize
	}
}

//////////////////////////////////////////////////////////////////////
//  Called send a 1024 byte ASCP formated message to the Client UDP Socket
//  Add a two byte sequence number to the header
//////////////////////////////////////////////////////////////////////
void CUSBsocket::SendUDPPacketToSocket( unsigned __int8* pData)
{
tASCPDatastruct DataMsg;
	if(m_UDPSocket == INVALID_SOCKET)
		return;
	if(!m_USBPortOpen)
		return;
	DataMsg.bytes.h0 = 0x04;	//data item0 length 0x404 = 1024+4
	DataMsg.bytes.h1 = 0x84;
	DataMsg.bytes.seql = m_SeqNumber&0xFF;
	DataMsg.bytes.seqh = (m_SeqNumber>>8)&0xFF;
	memcpy(DataMsg.buf+4, pData, 1024);
	int result = sendto(m_UDPSocket, (char*)DataMsg.buf, 1024+4, 0,
							(SOCKADDR *)&m_ClientUDPAddress,sizeof(m_ClientUDPAddress) );
	if(result != (1024+4) )
	{		// Something bad happened on the socket.  Deal with it.
		StopSDRxx();
		ShutdownConnection(m_ClientSocket);
		m_SocketSstatus = SOCKET_WAITINGCONNECT;
		m_USBReInit = TRUE;	//signal USB port so it can reinitialize
	}
	m_SeqNumber++;
	if( 0 == m_SeqNumber)	//if seq number rollover
		m_SeqNumber = 1;	//dont use zero as seq number
}

//////////////////////////////////////////////////////////////////////
//  Main processing loop of this IO object(run in worker thread)
// This is a worker thread that services the TCP/IP Socket.
//////////////////////////////////////////////////////////////////////
void CUSBsocket::SocketThread()
{
struct sockaddr_in listadr;
fd_set ReadFDs, ExceptFDs;
int nRet;
struct timeval timeout;
char RxBuf[MAX_RX];
	timeout.tv_sec =  2;		// select will wait for 2 seconds at a time
	timeout.tv_usec = 0;
	Sleep(1000);
	m_ListenSocket = INVALID_SOCKET;
	m_ClientSocket = INVALID_SOCKET;
	m_SocketSstatus = SOCKET_WAITINGUSB;
	memset( &m_ClientTCPAddress, 0, sizeof(m_ClientTCPAddress));
	memset( &m_ClientUDPAddress, 0, sizeof(m_ClientUDPAddress));

//Create socket
	if( (m_ListenSocket = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP) ) == INVALID_SOCKET)
	{
		nRet = ::WSAGetLastError();
		TRACE("Can't Open Socket %d\n", nRet);
		return;
	}
	memset( &listadr, 0, sizeof(listadr));
	listadr.sin_family = AF_INET;
	listadr.sin_addr.s_addr = htonl(INADDR_ANY);
	listadr.sin_port = htons(m_PortNum);
//Bind socket
	if (::bind(m_ListenSocket, (struct sockaddr *)&listadr, sizeof(struct sockaddr)) != 0)
	{
		nRet = ::WSAGetLastError();
		TRACE("Can't Bind address to Socket %d\n", nRet);
		return;
	}
	
	DWORD val = 1;
	ioctlsocket(m_ListenSocket, FIONBIO, &val);	//set non-blocking mode
	if( ::listen(m_ListenSocket, 1) == SOCKET_ERROR)
	{
		nRet = ::WSAGetLastError();
		TRACE("Can't Listen to Socket %d\n", nRet);
		return;
	}
	m_FiFoReset = FALSE;;
	while(m_SocketThreadRunning)
	{
		SetupFDSets(ReadFDs, ExceptFDs);
		int ret = select(0, &ReadFDs, NULL, &ExceptFDs, &timeout);
        if( ret > 0)
		{
			// Something happened on one of the sockets.
			// Was it the listener socket?...
			if( FD_ISSET(m_ListenSocket, &ReadFDs) )
			{
				FD_CLR(m_ListenSocket, &ReadFDs);
				int nAddrSize = sizeof(struct sockaddr_in);
				m_ClientSocket = accept(m_ListenSocket, (sockaddr*)&m_ClientTCPAddress, &nAddrSize);
                if (m_ClientSocket != INVALID_SOCKET)
				{
					// Mark the socket as non-blocking, to be sure.
					u_long nNoBlock = 1;
					ioctlsocket(m_ClientSocket, FIONBIO, &nNoBlock);
					TRACE("Accept Client Connection\n");
					m_SocketSstatus = SOCKET_CONNECTED;
					//now open the UDP socket
					memset( &m_ClientUDPAddress, 0, sizeof(m_ClientUDPAddress));
					m_ClientUDPAddress.sin_family = AF_INET;
					m_ClientUDPAddress.sin_addr = m_ClientTCPAddress.sin_addr;
					m_ClientUDPAddress.sin_port = htons(m_PortNum);
					m_FiFoReset = FALSE;;
					m_SeqNumber = 0;
				}
				else
				{
					TRACE("Accept Failed\n");
                    return;
                }
			}
			else if(FD_ISSET(m_ListenSocket, &ExceptFDs))
			{
				int err;
				int errlen = sizeof(err);
				getsockopt(m_ListenSocket, SOL_SOCKET, SO_ERROR,(char*)&err, &errlen);
				TRACE("Listen Exception Failure %d\n",err);
				return;
            }

			// ...Or was it the client socket?
			BOOL bOK = TRUE;
			// See if this client socket's flag is set in any of the FD sets.
			if (FD_ISSET(m_ClientSocket, &ExceptFDs))
			{
				bOK = FALSE;
				FD_CLR(m_ClientSocket, &ExceptFDs);
				TRACE("Client Exception\n");
				m_SocketSstatus = SOCKET_WAITINGCONNECT;
				m_USBReInit = TRUE;	//signal USB port so it can reinitialize
			}
			else
			{
				if (FD_ISSET(m_ClientSocket, &ReadFDs))
				{	//is a client read event
					FD_CLR(m_ClientSocket, &ReadFDs);
					int nBytes = recv(m_ClientSocket, RxBuf,MAX_RX , 0);
					if(nBytes<=0)
					{	//read event with no bytes means client closed socket
						ShutdownConnection(m_ClientSocket);
						m_SocketSstatus = SOCKET_WAITINGCONNECT;
						TRACE("Client Disconnect\n");
						m_USBReInit = TRUE;	//signal USB port so it can reinitialize
					}
					else
					{	//send client msg to SDR on USB port
						SendDataToUSB( RxBuf, nBytes);
						m_KeepAlive = 0;
//TRACE("\nmsgl %d",nBytes);
					}
				}
			}

			if(!bOK)
			{
				// Something bad happened on the socket, or the
				// client closed its half of the connection. 
				int err;
				int errlen = sizeof(err);
				getsockopt(m_ClientSocket, SOL_SOCKET, SO_ERROR,
						(char*)&err, &errlen);
				if (err != NO_ERROR)
				{
					TRACE("Client Error\n");
				}
				ShutdownConnection(m_ClientSocket);
				m_SocketSstatus = SOCKET_WAITINGCONNECT;
				m_USBReInit = TRUE;	//signal USB port so it can reinitialize
			}
		}
		else
		{	//here with a select error or timeout
			if(ret == SOCKET_ERROR)
			{
				TRACE("select error\n");
				return;
			}
			if( (m_ClientSocket!=INVALID_SOCKET) && !m_USBPortOpen )
			{
				ShutdownConnection(m_ClientSocket);
				TRACE("Client shutdown, USB link down\n");
			}
//			else
//				TRACE("Socket Timeout\n");
		}
	}
	ShutdownConnection(m_ClientSocket);
	ShutdownConnection(m_ListenSocket);
}

//////////////////////////////////////////////////////////////////////
// clears and sets the select events to wait for
//////////////////////////////////////////////////////////////////////
void CUSBsocket::SetupFDSets(fd_set& ReadFDs,  fd_set& ExceptFDs)
{
	FD_ZERO(&ReadFDs);
	FD_ZERO(&ExceptFDs);

// Add the listener socket to the read and except FD sets, if there
// is one. 
	if (m_ListenSocket != INVALID_SOCKET)
	{
		if(m_USBPortOpen)	//deactivate server if USB is down
		{
			if(m_ClientSocket==INVALID_SOCKET)
				m_SocketSstatus = SOCKET_WAITINGCONNECT;
			FD_SET(m_ListenSocket, &ReadFDs);
		}
		else
		{
			m_SocketSstatus = SOCKET_WAITINGUSB;
		}
		FD_SET(m_ListenSocket, &ExceptFDs);
	}

	// Add client connection events as needed
	if(	m_ClientSocket != INVALID_SOCKET )
	{
		FD_SET(m_ClientSocket, &ExceptFDs);
		FD_SET(m_ClientSocket, &ReadFDs);
	}
}

//////////////////////////////////////////////////////////////////////
// Gracefully shuts the connection sd down.  Returns true if we're
// successful, false otherwise.
//////////////////////////////////////////////////////////////////////
BOOL CUSBsocket::ShutdownConnection(SOCKET& sd)
{
	if(sd == INVALID_SOCKET)
		return FALSE;
	// Disallow any further data sends.  This will tell the other side
	// that we want to go away now.  If we skip this step, we don't
	// shut the connection down nicely.
	if(shutdown(sd, SD_SEND) == SOCKET_ERROR)
	{
		closesocket(sd);	//fails if is Listner socket so close here
		sd = INVALID_SOCKET;
		TRACE("Listener Shutdown\n");
		return FALSE;
	}

	// Receive any extra data still sitting on the socket.  After all
	// data is received, this call will block until the remote host
	// acknowledges the TCP control packet sent by the shutdown above.
	// Then we'll get a 0 back from recv, signalling that the remote
	// host has closed its side of the connection.
	char acReadBuffer[1500];
	while(1)
	{
		int nNewBytes = recv(sd, acReadBuffer, 1500, 0);
		if (nNewBytes == SOCKET_ERROR)
		{
			closesocket(sd);
			sd = INVALID_SOCKET;
			return FALSE;
		}
		else if(nNewBytes != 0)
		{
			TRACE("FYI, received %d unexpected bytes during shutdown.", nNewBytes);
		}
		else
		{	// Okay, we're done!
			break;
		}
	}

	// Close the socket.
	if(closesocket(sd) == SOCKET_ERROR)
	{
		sd = INVALID_SOCKET;
		TRACE("Socket Close Error\n");
		return FALSE;
	}
	sd = INVALID_SOCKET;
	TRACE("Socket Closed\n");
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
//sends an OFF command to the SDR-xx so it will stop sending data
//////////////////////////////////////////////////////////////////////
void CUSBsocket::StopSDRxx()
{
char txmsg[10];
	txmsg[0] = CI_RX_STATE_SETRESPLEN;
	txmsg[1] = TYPE_HOST_SET_CITEM;
	txmsg[2] = CI_RX_STATE;
	txmsg[3] = 0;
	txmsg[4] = 0;
	txmsg[5] = RX_STATE_IDLE;
	txmsg[6] = 0;
	txmsg[7] = 0;
	SendDataToUSB( txmsg, CI_RX_STATE_SETRESPLEN);
}


//////////////////////////////////////////////////////////////////////
// Sends a formatted ASCP msg to the USB device
//////////////////////////////////////////////////////////////////////
void CUSBsocket::SendASCPMsgToUSB( ASCPstruct* pASCPmsg)
{
	SendDataToUSB( (char*)pASCPmsg->buf, pASCPmsg->diwords.header & LENGTH_MASK);
}

//////////////////////////////////////////////////////////////////////
//decode some msgs for local use like name, serial number etc.
//////////////////////////////////////////////////////////////////////
void CUSBsocket::InterceptSdrMsgs( ASCPstruct* pASCPmsg, int length )
{
char * sptr;
	switch(pASCPmsg->words.CItem)
	{
		case CI_GENERAL_INTERFACE_NAME:
			sptr = (char*)pASCPmsg->bytes.parms;
			m_SdrName = sptr;
			break;
		case CI_GENERAL_INTERFACE_SERIALNUM:
			sptr = (char*)pASCPmsg->bytes.parms;
			m_SdrSN = sptr;
			break;
		case CI_GENERAL_HARDFIRM_VERSION:
			if( 0 == pASCPmsg->bytes.parms[0] )
			{
				m_BootVer = pASCPmsg->bytes.parms[2];
				m_BootVer <<= 8;
				m_BootVer |= pASCPmsg->bytes.parms[1];
			}
			else
			{
				m_FirmVer = pASCPmsg->bytes.parms[2];
				m_FirmVer <<= 8;
				m_FirmVer |= pASCPmsg->bytes.parms[1];
			}
			break;
		default:
			break;
	}
}


//////////////////////////////////////////////////////////////////////
//requests some info from the USB connected device
//////////////////////////////////////////////////////////////////////
void CUSBsocket::RequestDeviceInfo()
{
	m_TxASCPmsg.words.header = CI_GENERAL_INTERFACE_NAME_REQLEN;
	m_TxASCPmsg.bytes.b1 |= TYPE_HOST_REQ_CITEM;
	m_TxASCPmsg.words.CItem = CI_GENERAL_INTERFACE_NAME;
	SendASCPMsgToUSB(&m_TxASCPmsg);

	m_TxASCPmsg.words.header = CI_GENERAL_INTERFACE_SERIALNUM_REQLEN;
	m_TxASCPmsg.bytes.b1 |= TYPE_HOST_REQ_CITEM;
	m_TxASCPmsg.words.CItem = CI_GENERAL_INTERFACE_SERIALNUM;
	SendASCPMsgToUSB(&m_TxASCPmsg);


	m_TxASCPmsg.words.header = CI_GENERAL_HARDFIRM_VERSION_REQLEN;
	m_TxASCPmsg.bytes.b1 |= TYPE_HOST_REQ_CITEM;
	m_TxASCPmsg.words.CItem = CI_GENERAL_HARDFIRM_VERSION;
	m_TxASCPmsg.bytes.parms[0] = 0;
	SendASCPMsgToUSB(&m_TxASCPmsg);

	m_TxASCPmsg.words.header = CI_GENERAL_HARDFIRM_VERSION_REQLEN;
	m_TxASCPmsg.bytes.b1 |= TYPE_HOST_REQ_CITEM;
	m_TxASCPmsg.words.CItem = CI_GENERAL_HARDFIRM_VERSION;
	m_TxASCPmsg.bytes.parms[0] = 1;
	SendASCPMsgToUSB(&m_TxASCPmsg);

}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CString CUSBsocket::GetIPs( int &IP)
{
char ac[80];
CString name = "";
	if(gethostname(ac, sizeof(ac)) == SOCKET_ERROR)
	{
		return name;
	}
    name = ac;

	name += " - ";
struct hostent *pHost = gethostbyname(ac);
	if(NULL == pHost)
		return name;
char aszIPAddresses[3][16]; // maximum of three IP addresses
struct sockaddr_in SocketAddress;
	for(int iCnt = 0; ((pHost->h_addr_list[iCnt]) && (iCnt < 3)); ++iCnt)
	{
		memcpy(&SocketAddress.sin_addr, pHost->h_addr_list[iCnt], pHost->h_length);
		strcpy(aszIPAddresses[iCnt], inet_ntoa(SocketAddress.sin_addr));
		name += aszIPAddresses[iCnt];
		name += " ";
		m_CurrentIPAddress = htonl(SocketAddress.sin_addr.S_un.S_addr);		//save as little endian
	}
	return name;
}


